"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const RpcError_1 = require("./RpcError");
class JsonRpcClient {
    constructor(endPoint, credentials = 'include', newHeaders = null, debug = false) {
        this.defaultHeaders = {
            'Accept': 'application/json, text/plain, */*',
            'Content-Type': 'application/json',
            'X-Requested-With': 'XMLHttpRequest'
        };
        this.settings = {
            lastId: 1,
            endPoint: '',
            headers: this.defaultHeaders,
            credentials: null,
            debug: false
        };
        this.setNewHeaders = (newHeaders = {}) => {
            for (const [key, value] of Object.entries(newHeaders)) {
                this.settings.headers[key] = value;
            }
        };
        this.request = (method, params = {}) => {
            const req = {
                method: 'POST',
                credentials: this.settings.credentials,
                headers: this.settings.headers,
                body: JSON.stringify({
                    jsonrpc: '2.0',
                    id: this.settings.lastId++,
                    method: method,
                    params: params
                })
            };
            return fetch(this.settings.endPoint, req)
                .catch(error => {
                if (String(error).includes("Failed to fetch")) {
                    return location.reload();
                }
                throw error;
            })
                .then(res => this.checkStatus(res))
                .then(res => this.parseJSON(res))
                .then(res => this.checkError(res, req, this.settings.debug))
                .then(res => this.logResponse(res, this.settings.debug));
        };
        this.settings.endPoint = endPoint;
        this.settings.credentials = credentials;
        this.settings.debug = debug;
        if (newHeaders !== null) {
            for (const [key, value] of Object.entries(newHeaders)) {
                this.settings.headers[key] = value;
            }
        }
    }
    parseJSON(response) {
        return response.json();
    }
    checkStatus(response) {
        // we assume 400 as valid code here because it's the default return code when sth has gone wrong,
        // but then we have an error within the response, no?
        if (response.status >= 200 && response.status <= 400) {
            return response;
        }
        const error = new Error(response.statusText);
        throw error;
    }
    checkError(data, req, debug = false) {
        if (data.error) {
            /* eslint-disable no-console */
            if (debug === true && console && console.error) {
                console.error(`Request ID ${data.id} failed: ${data.error}`);
            }
            else if (debug === true && console && console.log) {
                console.log(`Request ID ${data.id} failed: ${data.error}`);
            }
            /* eslint-enable no-console */
            throw new RpcError_1.RpcError(data.error.message, req, data.error.data, data.error.code);
        }
        return data;
    }
    logResponse(response, debug = false) {
        if (debug === true) {
            /* eslint-disable no-console */
            console.log('Got response for id', response.id, 'with response', response.result);
            console.log('Response message for request', response.id, ':', response.result.message);
            /* eslint-enable no-console */
        }
        return response.result;
    }
}
exports.default = JsonRpcClient;
